# frequencywidget.py -- widget used in the frequency tab
#
# Copyright (c) Adam Tenderholt, atenderholt@gmail.com, 2014
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

from math import sin, pi
from operator import attrgetter
import os

import numpy
from PyQt4 import QtCore, QtGui, Qwt5

from ui_frequency_widget import Ui_frequencyWidget

NPOINTS = 10
INDICES = range(NPOINTS) + range(-1 * NPOINTS, NPOINTS + 1)[::-1] + range(-1 * NPOINTS + 1, 0)
current = { QtCore.QString("IR Spectrum"): attrgetter("vibirs"), 
            QtCore.QString("Raman Spectrum"): attrgetter("vibramans") }

def lorentzian(xvalues, freq, intensity, width):
    a = width**2.0 / 4.0
    deltas = (xvalues - freq)**2
    return intensity * a / (deltas + a)

class FrequencyWidget(QtGui.QWidget, Ui_frequencyWidget):
    def __init__(self, parent=None):
        super(FrequencyWidget, self).__init__(parent)

        self.setupUi(self)
        self.interval = 50
        self.value = 0
        self.animated = False
        self.slider.setMinimum(-1 * NPOINTS)
        self.slider.setMaximum(NPOINTS)

        # main curve
        self.curve = Qwt5.QwtPlotCurve()
        self.curve.attach(self.plot)

        # selected stick
        self.stick = Qwt5.QwtPlotCurve()
        self.stick.setStyle(Qwt5.QwtPlotCurve.Sticks)
        pen = QtGui.QPen(QtGui.QColor("red"))
        pen.setWidth(2)
        self.stick.setPen(pen)
        self.stick.attach(self.plot)

        # all sticks
        self.sticks = Qwt5.QwtPlotCurve()
        self.sticks.setStyle(Qwt5.QwtPlotCurve.Sticks)

        self.xvalues = numpy.arange(0, 4000.1, 1.0)
        self.plot.setAxisScale(Qwt5.QwtPlot.xBottom, 0, 4000)

        # create picker/zoomer
        self.zoomer = Qwt5.QwtPlotZoomer(self.plot.canvas())
        self.zoomer.setSelectionFlags(Qwt5.QwtPicker.DragSelection | Qwt5.QwtPicker.RectSelection)
        self.zoomer.setTrackerMode(Qwt5.QwtPicker.AlwaysOff)
        self.picker = Qwt5.QwtPlotPicker(self.plot.canvas())
        self.picker.setTrackerMode(Qwt5.QwtPicker.AlwaysOn)
        self.picker.setSelectionFlags(Qwt5.QwtPicker.ClickSelection | Qwt5.QwtPicker.PointSelection)

        # connect picker/zoomer
        QtCore.QObject.connect(self.picker, QtCore.SIGNAL("selected(const QwtDoublePoint&)"), 
                                self.slotPickedPoint)
        QtCore.QObject.connect(self.zoomer, QtCore.SIGNAL("zoomed(const QwtDoubleRect&)"), 
                                self.slotZoomed)

    def setMainWindowActions(self, saveAnimationAction, saveStructureAction):
        self.saveAnimationAction = saveAnimationAction
        self.saveStructureAction = saveStructureAction

    def slotZoomed(self, rect):
        width = rect.right() - rect.left()
        if width <= 1000:
            self.sticks.attach(self.plot)
        else:
            self.sticks.detach()

    def setData(self, data):
        self.data = data

        # update table and associated view/selection
        self.model = FrequencyModel(data, self.frequencyView)
        self.frequencyView.setModel(self.model)
        self.selection = QtGui.QItemSelectionModel(self.model)
        self.selection.select(self.model.index(0, 0), QtGui.QItemSelectionModel.Select)
        self.frequencyView.setSelectionModel(self.selection)
        self.frequencyView.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.frequencyView.resizeColumnsToContents()

        # populate combo box
        self.comboBox.clear()
        if hasattr(self.data, "vibirs"):
            self.comboBox.addItem("IR Spectrum")
        if hasattr(self.data, "vibramans"):
            self.comboBox.addItem("Raman Spectrum")

        QtCore.QObject.connect(self.selection, 
                                QtCore.SIGNAL("selectionChanged( const QItemSelection, \
                                const QItemSelection)"), 
                                self.changeSelection)

        self.updateMolecularFrames(0)

    def updateMolecularFrames(self, index):

        self.glWidget.reset()
        self.glWidget.appendMolecule(self.data.atomcoords[-1], self.data.atomnos)
        displacement = self.displacementSpinBox.value()
        for i in range(-1 * NPOINTS, NPOINTS + 1):
            value = sin(i * pi / (NPOINTS * 2) )
            newcoords = self.data.atomcoords[-1] + \
                                displacement * value * self.data.vibdisps[index]
            self.glWidget.insertMolecule(i, newcoords, self.data.atomnos)

    def beginAnimation(self):
        self.timer = self.startTimer(self.interval)
        self.animated = True
        self.pushButton.setText("Stop")
        self.slider.setEnabled(False)
        self.saveAnimationAction.setEnabled(True)
        self.saveStructureAction.setEnabled(False)

    def endAnimation(self):
        if hasattr(self, "timer") and self.timer:
            self.killTimer(self.timer)

        self.animated = False
        self.pushButton.setText("Start")
        self.slider.setEnabled(True)
        self.saveAnimationAction.setEnabled(False)
        self.saveStructureAction.setEnabled(True)

        self.slider.setValue(INDICES[self.value])

    def animateButtonClicked(self):
        if self.animated:
            self.endAnimation()
        else:
            self.beginAnimation()

    def updateSpectrumType(self, type):
        self.updateSpectrum(current[type](self.data))

    def updateSpectrum(self, intensities):
        freqs = self.data.vibfreqs
        assert len(freqs) == len(intensities)

        width = 10
        xvalues = self.xvalues
        yvalues = numpy.zeros((len(xvalues),))
        factor = self.scaleFactorSpinBox.value()
        for freq, intensity in zip(freqs * factor, intensities):
            yvalues += lorentzian(xvalues, freq, intensity, width)

        self.curve.setData(xvalues, yvalues)
        self.sticks.setData(freqs * factor, intensities)

        # update plot and zoomer
        self.plot.setAxisScale(Qwt5.QwtPlot.yLeft, 0, 1.1 * max(yvalues))
        self.zoomer.setZoomBase(QtCore.QRectF(0.0, 0.0, 4000.0, 1.1 * max(yvalues)))
        self.plot.replot()

        self.yvalues = yvalues

    def updateScaleFactor(self, value):
        self.updateSpectrumType(self.comboBox.currentText())
        self.model.setScaleFactor(value)
        self.updateSticks()

    def updateSticks(self):
        indexes = self.selection.selectedIndexes()
        if len(indexes) == 1:
            index = indexes[0].row()

            type = self.comboBox.currentText()
            intensity = current[type](self.data)[index]
            freq = self.data.vibfreqs[index] * self.scaleFactorSpinBox.value()
            self.stick.setData([freq], [intensity])
            self.plot.replot()

    def updateDisplacement(self, value):
        indexes = self.selection.selectedIndexes()
        if len(indexes) == 1:
            index = indexes[0].row()
            self.updateMolecularFrames(index)

    def slotPickedPoint(self, point):
        xvalues = self.data.vibfreqs * self.scaleFactorSpinBox.value()
        delta = numpy.abs(xvalues - point.x())
        index = numpy.argmin(delta)
        self.frequencyView.setCurrentIndex(self.model.index(index, 0))

    def changeSelection(self, selected, deselected):
        indexes = selected.indexes()
        
        if len(indexes) == 1: # assume only one row selected
            index = indexes[0].row()
            self.updateMolecularFrames(index)
            self.updateSticks()

    def timerEvent(self, event):
        if not hasattr(self, "data"):
            return

        self.value += 1
        if self.value >= len(INDICES):
            self.value = 0
        value = INDICES[self.value]

        self.setCurrentValue(value)

    def setCurrentValue(self, value):
        self.glWidget.current = value
        self.glWidget.updateGL()

    def getCurrentFrequency(self):
        # get scaled wavenumber of the currently selected vibration
        indexes = self.selection.selectedIndexes()
        row = indexes[0].row()
        factor = self.scaleFactorSpinBox.value()
        wavenumber = self.data.vibfreqs[row] * factor
        return wavenumber

    def saveAnimation(self, filename):

        from PIL import Image
        from images2gif import writeGif
        import glob, os
        import time

        should_resume = False
        if self.animated:
            self.endAnimation()
            should_resume = True

        # setup progress dialog
        steps = len(INDICES)
        dialog = QtGui.QProgressDialog("Rendering frames for animation...", "Cancel", 0, steps * 2, 
                                        self)
        dialog.setWindowModality(QtCore.Qt.WindowModal)
        dialog.setMinimumDuration(1000)

        images = []
        for i, index in enumerate(INDICES):
            dialog.setValue(i)

            if dialog.wasCanceled():
                break

            self.glWidget.current = index
            image = self.glWidget.renderFrame()
            temp_name = "frame_%02i.bmp" % i
            image.save(temp_name)
            images.append(Image.open(temp_name))

        dialog.setLabelText("Saving GIF...")
        QtGui.QApplication.processEvents()

        if len(images) == len(INDICES):
            writeGif(filename, images, self.interval / 1000.0)

        files = glob.glob("frame*bmp")
        for file in files:
            os.remove(file)

        dialog.setValue(steps * 2)
        if should_resume:
            self.beginAnimation()

    def saveSpectrum(self, filename):
        
        factor = self.scaleFactorSpinBox.value()
        freqs = self.data.vibfreqs * factor
        type = self.comboBox.currentText()
        intensities = current[type](self.data)

        assert len(freqs) == len(intensities)

        # check to see if we have a name
        if filename:
            suggest = os.path.splitext(filename)[0] + ".txt"
            filename = QtGui.QFileDialog.getSaveFileName(self, "Save frequency results...", \
                                    suggest, "*.txt")

        else:
            newfilename = "newfile.txt"
            filename = QtGui.QFileDialog.getSaveFileName(self, "Save frequency results...", \
                                    newfilename, "*.txt")

        if not filename:
            return

        file = open(str(filename), "w")

        # write out a header
        file.write("# Frequencies scaled by %.4f\n" % factor)
        header = " Freqs.\t%s\t Freqs.\t%s Intensity\n" % (str(type), str(type).split()[0])
        file.write(header)

        size = len(freqs)
        xvalues = self.xvalues
        yvalues = self.yvalues

        # now write out data, sticks
        temp = numpy.column_stack([ xvalues[:size], yvalues[:size], freqs, intensities])
        numpy.savetxt(file, temp, fmt="%7.2f\t%9.3f\t%7.2f\t%9.3f")
        temp = numpy.column_stack([ xvalues[size:], yvalues[size:]])
        numpy.savetxt(file, temp, fmt="%7.2f\t%9.3f")

        file.close()

class FrequencyModel(QtCore.QAbstractTableModel):
    def __init__(self, data, parent=None):
        super(FrequencyModel, self).__init__(parent)

        self.data = data
        self.factor = 1.0

    def headerData(self, section, orientation, role):
        if role != QtCore.Qt.DisplayRole:
            return None

        if orientation == QtCore.Qt.Horizontal:
            return "Frequency"

        if orientation == QtCore.Qt.Vertical:
            return section + 1

    def rowCount(self, parent):
        if not hasattr(self.data, "vibfreqs"):
            return 0

        return len(self.data.vibfreqs)

    def columnCount(self, parent):
        return 1

    def data(self, index, role):
        if role != QtCore.Qt.DisplayRole:
            return None

        if not self.data:
            return None

        if index.column() == 0 and hasattr(self.data, "vibfreqs"):
            value = self.data.vibfreqs[index.row()] * self.factor
            return "%i" % value

        return None

    def setScaleFactor(self, factor):
        self.factor = factor
        topLeft = self.index(0, 0)
        bottomRight = self.index(len(self.data.vibfreqs), 0)

        self.emit(QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"), topLeft,
                    bottomRight)

if __name__ == "__main__":
    import sys
    from cclib.parser import ccopen

    parser = ccopen(sys.argv[1])
    data = parser.parse()
    app = QtGui.QApplication(sys.argv)
    widget = FrequencyWidget()
    widget.show()
    widget.setData(data)
    widget.beginAnimation()

    result = app.exec_()
    sys.exit(result)

