
# popitems.py -- class for managing atoms and orbitials in the popwidget
#
# Copyright (c) Adam Tenderholt, atenderholt@gmail.com, 2012-2014
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.
#
# This file is based on the simpletreemodel example distributed
# with PyQt4-mac-gpl-4.9.1. 

ROOT, ATOM, FRAGMENT, ORBITAL = range(4)

# Atoms have numbers and names
# Fragments have numbers and names
NUMBER = 0
NAME = 1

# Orbitals also have atom names and numbers
ATOM_NUMBER = 2
ATOM_NAME = 3

class PopItem(object):
    def __init__(self, data, parent=None, type=ROOT):
        self._type = type
        self._parent = parent
        self._children = []

        # use dict instead of array so that items can be referred to by name
        self._data = data

    def columnCount(self):
        return 1 

    def childCount(self):
        return len(self._children)

    def childIndex(self, child):
        if child in self._children:
            return self._children.index(child)
        else:
            return -1

    def appendChild(self, child):
        self._children.append(child)

    def appendChildren(self, children):
        for child in children:
            self.appendChild(child)

    def insertChild(self, index, child):
        child.setParent(self)
        self._children.insert(index, child)

    def insertChildren(self, index, children):
        for child in children[::-1]:
            self.insertChildren(index, child)

    def removeChild(self, child):
        index = self.childIndex(child)
        return self.removeChildByIndex(index)

    def removeChildByIndex(self, index):
        if index >= 0:
            child = self._children.pop(index)
            return child
        else:
            return None
        
    def removeChildren(self, children):
        for child in children:
            index = self.childIndex(child)
            if index >= 0:
                self._children.pop(index)

    def parent(self):
        return self._parent

    def setParent(self, parent):
        self._parent = parent

    def child(self, index):
        if index >= self.childCount():
            return None
        else:
            return self._children[index]

    def data(self, column):
        try:
            value = self._data[column]
        except KeyError:
            print "Execption in TreeItem.data(): %i" % column, self._data
            return None

        else: 
            return value

    def setData(self, column, value):
        self._data[column] = value

    def type(self):
        return self._type

    def row(self):
        if self._parent:
            return self._parent.childIndex(self)

        else:
            return -1

    def representedAsDictionary(self):
        dictionary = {}
        dictionary["data"] = self._data
        dictionary["type"] = self._type

        dictionary["children"] = []
        for child in self._children:
            dictionary["children"].append(child.representedAsDictionary())
        
        return dictionary
        
def buildTreeFromAONAMES(aonames):
    data_dict = {}
    data_dict[NUMBER] = 0
    data_dict[NAME] = "Root"

    rootItem = PopItem(data_dict)

    atom_names = []
    for i in range(len(aonames)):
        split_index = aonames[i].rfind("_")
        atom_name = aonames[i][:split_index]
        ao_name = aonames[i][split_index + 1:]
        
        # check if recent atom_name is in atom_names
        # if not, create new pop item
        if not atom_name in atom_names:
            atom_names.append(atom_name)
            data_dict = {}
            data_dict[NUMBER] = len(atom_names)
            data_dict[NAME] = atom_name

            atomItem = PopItem(data_dict, rootItem, ATOM)
            rootItem.appendChild(atomItem)

        data_dict = {}
        data_dict[NAME] = ao_name
        data_dict[NUMBER] = i + 1
        data_dict[ATOM_NUMBER] = len(atom_names)
        data_dict[ATOM_NAME] = atom_name

        orbitalItem = PopItem(data_dict, atomItem, ORBITAL)
        atomItem.appendChild(orbitalItem)

    return rootItem
    
if __name__ == "__main__":
    import sys
    from cclib.parser import ccopen

    parser = ccopen(sys.argv[1])
    data = parser.parse()

    root = buildTreeFromAONAMES(data.aonames)
    print "Root has %i children" % root.childCount()


