
# popmodels.py -- models for atomic and fragment orbitals
#
# Copyright (c) Adam Tenderholt, Stanford University, 2008
#                                  a-tenderholt@stanford.edu
#                                University of Washington, 2010
#                                  atenderholt@gmail.com
# Copyright (c) Adam Tenderholt (atenderholt@gmail.com), 2012-2014
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.
#
# This file is based on the simpletreemodel example distributed
# with PyQt4-mac-gpl-4.9.1. The original copyright noticed is reproduced
# below.

"""***************************************************************************
**
** Copyright (C) 2005-2005 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
***************************************************************************"""

import simplejson
from PyQt4 import QtCore, QtGui
from popitems import *

class PopModel(QtCore.QAbstractItemModel):
    def __init__(self, name, parent=None):
        super(PopModel, self).__init__(parent)

        data_dict = { NAME: "ROOT", NUMBER: -1 }
        self.name = name
        self.rootItem = PopItem(data_dict)
        self.setSupportedDragActions(QtCore.Qt.MoveAction)

    def headerData(self, section, orientation, role):
        if role != QtCore.Qt.DisplayRole:
            return None

        if orientation == QtCore.Qt.Horizontal:
            return self.name

        if orientation == QtCore.Qt.Vertical:
            return section

    def buildTreeFromAONAMES(self, aonames):
        self.rootItem = buildTreeFromAONAMES(aonames)
        self.reset() # needed to draw the tree after adding data?

    def setRootItem(self, rootItem):
        self.rootItem = rootItem

    def columnCount(self, parent):
        return 1

    def data(self, index, role):
        if not index.isValid() or index.column() > 0:
            return None

        if role == QtCore.Qt.EditRole:
            return  index.internalPointer().data(NAME)

        if role != QtCore.Qt.DisplayRole:
            return None

        item = index.internalPointer()
        type = item.type()


        if type == ATOM or type == FRAGMENT:
            return "(%i) %s" % (item.data(NUMBER), item.data(NAME))

        parentItem = item.parent()
        type = parentItem.type()

        if type == ATOM:
            return "(%i) %s" % (item.data(NUMBER), item.data(NAME))
        
        if type == FRAGMENT:
            return "(%i) %s:%s" % (item.data(NUMBER), item.data(ATOM_NAME), item.data(NAME))

        return ""

    def setData(self, index, value, role):
        if not index.isValid() or index.column() > 0:
            return False

        item = index.internalPointer()
        if item.type() != FRAGMENT:
            return False

        item.setData(NAME, value.toString())
        self.dataChanged.emit(index, index)

        return True

    def flags(self, index):
        if not index.isValid() or index.column() > 0:
            return QtCore.Qt.NoItemFlags

        item = index.internalPointer()
        type = item.type()

        if type == ATOM:
            return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable \
                    | QtCore.Qt.ItemIsDragEnabled 
        
        if type == FRAGMENT:
            return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable \
                    | QtCore.Qt.ItemIsDropEnabled | QtCore.Qt.ItemIsEditable

        if type == ORBITAL:
            return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable \
                    | QtCore.Qt.ItemIsDragEnabled

        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable

    def supportedDropActions(self):
        return QtCore.Qt.MoveAction

    def index(self, row, column, parent):
        if not self.hasIndex(row, column, parent):
            return QtCore.QModelIndex()

        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()

        childItem = parentItem.child(row)
        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QtCore.QModelIndex()

    def parent(self, index):
        if not index.isValid():
            return QtCore.QModelIndex()

        childItem = index.internalPointer()
        parentItem = childItem.parent()

        if parentItem == self.rootItem:
            return QtCore.QModelIndex()

        return self.createIndex(parentItem.row(), 0, parentItem)

    def rowCount(self, parent):
        if parent.column() > 0:
            return 0

        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()

        return parentItem.childCount()

    def mimeData(self, indices):
        mimeData = QtCore.QMimeData()
        indices_list = []
        items = []
        for index in indices:
            if index.column() > 0:
                continue
            item = index.internalPointer()
            type = item.type()
            if type == ORBITAL or type == FRAGMENT:
                item_rep = item.representedAsDictionary()
                if item_rep not in items:
                    items.append( item_rep )
            elif type == ATOM:
                for i in range(item.childCount()):
                    child = item.child(i)
                    child_rep = child.representedAsDictionary()
                    if child_rep not in items:
                        items.append( child.representedAsDictionary() )

        text = simplejson.dumps(items)
        mimeData.setData("application/x-qabstractitemmodeldatalist", text)

        return mimeData

    def dropMimeData(self, mimeData, action, row, column, parent):
        # note: if no peaks in series, row == column == -1

        if not parent.isValid():
            print "Dropping on root?"
            return False

        parentItem = parent.internalPointer()
        object_str = mimeData.data("application/x-qabstractitemmodeldatalist")
        object_list = simplejson.loads(str(object_str))

        size = len(object_list)
        if row == -1:
            row = parentItem.childCount()
#            self.beginInsertRows(parent, parentItem.childCount(), parentItem.childCount() + size)

        self.beginInsertRows(parent, row, row + size - 1)

        for i in range(size):
            data_dict = {}
            for key, value in object_list[i]["data"].iteritems():
                data_dict[int(key)] = value
            childItem = PopItem(data_dict, parentItem, object_list[i]["type"])
            parentItem.insertChild(row + i, childItem)

        self.endInsertRows()

        # fix triangle indicators
        index = self.index(parentItem.row(), 0, QtCore.QModelIndex())
        self.dataChanged.emit(index, index)

        return True

    def cleanupEmptyAtoms(self):
        # iterate over atoms backwards, checking whether still has orbitals
        # if not, delete
        for i in range(self.rootItem.childCount())[::-1]:
            atom = self.rootItem.child(i)
            if atom.childCount() > 0:
                continue
            print i
            self.removeRows(i, 1, QtCore.QModelIndex())

        return
        
    def removeRows(self, row, count, parent):
        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()

        self.beginRemoveRows(parent, row, row + count - 1)

        for i in range(count):
            child = parentItem.removeChildByIndex(row)
            del child

        self.endRemoveRows()
        return True

    def addFragment(self):
        newPlotName = "New Fragment"
        self.addFragmentByName(newPlotName)

    def addFragmentByName(self, name):

        self.beginInsertRows(QtCore.QModelIndex(), self.rootItem.childCount(), 
                self.rootItem.childCount())

        # setup data dictionary for tree item
        data_dict = {}
        data_dict[NAME] = name
        data_dict[NUMBER] = self.rootItem.childCount() + 1

        # create new PopItem and add it to the root
        plotItem = PopItem(data_dict, self.rootItem, FRAGMENT)
        self.rootItem.appendChild(plotItem)

        self.endInsertRows()

        return

    def getFragments(self):
        if self.name != "Fragments/Orbitals":
            print "Name doesn't match in getFragments. Returning None, None"
            return None, None

        names = []
        lists = []

        for i in range(self.rootItem.childCount()):
            child = self.rootItem.child(i)
            list = []
            for j in range(child.childCount()):
                list.append(child.child(j).data(NUMBER) - 1)

            if len(list) > 0:
                lists.append(sorted(list))
                names.append(child.data(NAME))

        return names, lists

    def updateTree(self):
        # use this function to fix numbers when fragments are removed

        for i in range(self.rootItem.childCount()):
            child = self.rootItem.child(i)
            child.setData(NUMBER, child.row() + 1)

if __name__ == "__main__":
    import sys
    from cclib.parser import ccopen

    app = QtGui.QApplication(sys.argv)

    # parse cclib object
    parser = ccopen(sys.argv[1])
    data = parser.parse()

    # setup atom model
    model1 = PopModel("Atoms/Orbitals")
    model1.buildTreeFromAONAMES(data.aonames)

    # setup atom view
    view1 = QtGui.QTreeView()
    view1.setModel(model1)
    view1.setWindowTitle("Simple Tree Model")
    view1.setDragEnabled(True)
    view1.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
    view1.show()

    # setup fragment popmodel
    model2 = PopModel("Fragments/Orbitals")
    model2.addFragment()
    model2.addFragment()

    # setup fragments view
    view2 = QtGui.QTreeView()
    view2.setModel(model2)
    view2.setWindowTitle("Drop")
    view2.setDragEnabled(True)
    view2.setAcceptDrops(True)
    view2.setDropIndicatorShown(True)
    view2.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
    view2.show()

    sys.exit(app.exec_())

