# popwidget.py -- widget used in the population methods tab
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2007
#                               a-tenderholt@stanford.edu
# Copyright (c) Adam Tenderholt, atenderholt@gmail.com, 2012-2014
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

from PyQt4 import QtCore, QtGui

from popmodel import *
from glwidget import *
from dialogs import SaveFormatDialog

from cclib.parser import utils

table = utils.PeriodicTable()

class PopWidget(QtGui.QWidget):
    def __init__(self, parent = None, flags = 0):
        QtGui.QWidget.__init__(self)

        self.setParent(parent)
        self.resize(800, 600)

#setup widgets
        self.gridlayout = QtGui.QGridLayout(self)
        self.gridlayout.setMargin(9)
        self.gridlayout.setSpacing(6)
        self.gridlayout.setObjectName("gridlayout2")

        self.splitter = QtGui.QSplitter(self)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")

        hboxlayout = QtGui.QHBoxLayout(self.splitter)

#setup Tree views
        atomview = QtGui.QTreeView(self.splitter)
        atomview.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        atomview.setDragEnabled(True)
        atomview.show()

        fragview = QtGui.QTreeView(self.splitter)
        fragview.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        fragview.setDragEnabled(True)
        fragview.setAcceptDrops(True)
        fragview.setDropIndicatorShown(True)
        fragview.show()

#setup models
        self.aomodel = PopModel("Atoms/Orbitals")
        atomview.setModel(self.aomodel)
        self.selectionModel = QtGui.QItemSelectionModel(self.aomodel)
        atomview.setSelectionModel(self.selectionModel)
        self.fragmodel = PopModel("Fragments/Orbitals")
        fragview.setModel(self.fragmodel)

        hboxlayout.addWidget(atomview)
        hboxlayout.addWidget(fragview)

        glWidget = GLWidget(self.splitter)

        self.splitter.setSizes([75,75,250])
        self.gridlayout.addWidget(self.splitter)

#setup slots
        QtCore.QObject.connect(self.selectionModel,QtCore.SIGNAL("selectionChanged(const QItemSelection&, const QItemSelection&)"), self.changeSelection)

#setup class variables
        self.atomview = atomview
        self.fragview = fragview
        self.glWidget = glWidget
        self.data = None
        self.names = None

    def reset(self):
        self.fragview.reset()
        self.atomview.reset()

        rows = self.fragmodel.rowCount(QtCore.QModelIndex())
        self.fragmodel.removeRows(0, rows, QtCore.QModelIndex())

    def setData(self, data):
        
        if not ( hasattr(data,"fonames") or hasattr(data,"aonames") ):
            return

        if hasattr(data, "fonames"):
            self.names = data.fonames
            self.aomodel.buildTreeFromAONAMES(data.fonames)

        if hasattr(data, "aonames"):
            self.names = data.aonames
            self.aomodel.buildTreeFromAONAMES(data.aonames)

        if hasattr(data, "atomnos") and hasattr(data,"atomcoords"):
            self.glWidget.appendMolecule(data.atomcoords[0], data.atomnos)

        self.atomview.setModel(self.aomodel)

        self.data = data

    def changeSelection(self, current, old):
        selection = {}
        isADF = ( str(self.data).split()[0] == "ADF" )

        for index in self.selectionModel.selectedIndexes():
            parentIndex = index.parent()

            if parentIndex == QtCore.QModelIndex():
                text = self.aomodel.data(index, QtCore.Qt.DisplayRole)
            else:
                text = self.aomodel.data(parentIndex, QtCore.Qt.DisplayRole)
            
            text = QtCore.QString(text)
            lindex = text.indexOf(")")
            infos = text[lindex+2:].split("+", QtCore.QString.SkipEmptyParts)

            regexp = QtCore.QRegExp("[1-9]")

            for info in infos:
                if not isADF:
                    index = regexp.indexIn(info)
                    value = int(info[index:])
                elif self.isElement(str(info[:2])):
                    value = int(info[2:])
                elif self.isElement(str(info[0])):
                    value = int(info[1:])
                else:
                    for fragname in self.data.fragnames:
                        name = fragname.split("_")[0]
                        index = info.indexOf(name)
                        if index == 0:
                            value = int(info[len(name)])

                selection[value - 1] = 1

        if isADF:
            selectionList = []
            for key in selection.keys():
                selectionList += self.data.frags[key]
            
            self.glWidget.setSelected(selectionList)
        else:
            self.glWidget.setSelected(selection.keys())

    def isElement(self, text):
        try:
            table.number[text]
        except KeyError:
            return False
        else:
            return True

    def removeFragments(self):
        selectedIndices = self.fragview.selectedIndexes()
        for index in selectedIndices:
            if not index.isValid():
                continue

            item = index.internalPointer()
            if item.childCount() > 0:
                continue

            self.fragmodel.removeRows(item.row(), 1, QtCore.QModelIndex())

        self.fragmodel.updateTree()

if __name__=='__main__':
    import sys
    from os.path import exists
    from cclib.parser import ccopen

    app = QtGui.QApplication(sys.argv)
    window = PopWidget()
    window.show()

    if len(sys.argv)==2 and exists(sys.argv[1]):
       parser = ccopen(sys.argv[1])
       data = parser.parse()
       window.setData(data)
       window.fragmodel.addFragment()
       window.fragmodel.addFragment()

    sys.exit(app.exec_())

